import React, { ChangeEvent, useEffect, useState } from "react";
import { ListGroup } from "react-bootstrap";
import { Link } from "react-router-dom";
import { Paper } from "@material-ui/core";
import {
  Chart,
  PieSeries,
  Title,
  Tooltip,
} from "@devexpress/dx-react-chart-material-ui";
import { EventTracker } from "@devexpress/dx-react-chart";
import { Loading } from "..";
import axios from "axios";

const amount = (amount: number) => (amount / 1000000).toFixed(2);
const chains = ["Antioch", "Alexandria"];
const proposalsUrl =
  "https://raw.githubusercontent.com/Joystream/community-repo/master/documentation/spending_proposal_categories.json";

const executionFailed = (result: string, executed: any) => {
  if (result !== "Approved") return result;
  if (!executed || !Object.keys(executed)) return;
  if (executed.Approved && executed.Approved.ExecutionFailed)
    return executed.Approved.ExecutionFailed.error;
  return false;
};

interface Proposal {
  Testnet: string;
  "Proposal #": number;
  Title: string;
  Status: string;
  "Amount Asked": number;
  "Amount Minted": number;
  "Exchange Rate": number;
  "$ amount": number;
  Category: string;
  Bounty: number;
  Reimbursed: string;
}

interface Categories {
  [key: string]: SpendingProposal[];
}

const Spending = (props: IState) => {
  const [spending, setSpending] = useState([]) as Proposal[];
  const [chain, setChain] = useState("Antioch");
  const [tooltipItem, setTooltipItem] = useState(0);

  const selectChain = (event: ChangeEvent<{ value: string }>) => {
    setChain(event.target.value);
  };

  const getCategories = (proposals: Proposal[]): Categories => {
    let categories = {};
    proposals.forEach((p) => {
      if (p.Category.length)
        categories[p.Category] = categories[p.Category]
          ? categories[p.Category].concat(p)
          : [p];
    });
    return categories;
  };

  // iterate over category keys and proposals to calculate sums
  const calculateSums = (categories: Categories) => {
    let sorted = [];
    let total = 0;
    Object.keys(categories).forEach((category) => {
      const proposals = categories[category];
      let sum = 0;
      proposals.forEach((p) => {
        const amount = +p["Amount Minted"] / 1000000;
        sum += amount;
        total += amount;
      });
      sorted.push({ category, sum, proposals });
    });

    // add percent
    sorted = sorted.map((c) => {
      c.percent = (100 * (c.sum / total)).toFixed();
      return c;
    });

    return sorted.filter((c) => c.sum).sort((a, b) => b.sum - a.sum);
  };

  useEffect(() => {
    axios
      .get(proposalsUrl)
      .then(({ data }) => setSpending(data.filter((p) => p.Status.length)));
  }, []);

  if (!spending.length) return <Loading target="proposals" />;
  const filtered = spending.filter((p) => p.Testnet === chain);
  const categories = calculateSums(getCategories(filtered));

  return (
    <Paper className="p-2">
      <h2>Spending Proposals</h2>
      <select
        name="chain"
        className="form-control"
        value={chain}
        defaultValue={chain}
        onChange={selectChain}
      >
        {chains.map((c: string) => (
          <option key={c}>{c}</option>
        ))}
      </select>

      <div className="mx-3 d-flex flex-row align-items-center">
        <div className="col-6 flex-grow-1 mt-3 pt3 text-right">
          {categories.map(
            (c) =>
              c && (
                <div className="d-flex flex-row m-2">
                  <div className="col-5 font-weight-bold">{c.category}</div>
                  <div className="col-3">{c.sum.toFixed(1)} M</div>
                  <div className="col-3">{c.percent} %</div>
                </div>
              )
          )}
        </div>
        <div className="col-6">
          <Chart data={categories}>
            <PieSeries valueField="sum" argumentField="category" />
            <Title text="M tJOY" />
            <EventTracker />
            <Tooltip
              targetItem={tooltipItem}
              onTargetItemChange={setTooltipItem}
            />
          </Chart>
        </div>
      </div>

      <h2>{chain}</h2>
      <ListGroup>
        {filtered.map((p: any, i: number) => (
          <ListGroup.Item
            key={i}
            className={
              p["Amount Minted"]
                ? p.Reimbursed === "Y"
                  ? "bg-success"
                  : "bg-warning"
                : "bg-danger"
            }
          >
            <Link to={`/proposals/${p["Proposal #"]}`}>
              <div className="d-flex flex-row">
                <div className="col-1">{p["Proposal #"]}</div>
                <div className="col-4">{p.Title}</div>
                <div className="col-1">{p["Amount Asked"]}</div>
                <div className="col-2">{p.Category + " " + p.Bounty}</div>
                <div className="col-2">Reimbursed: {p.Reimbursed}</div>
              </div>
            </Link>
          </ListGroup.Item>
        ))}
      </ListGroup>
    </Paper>
  );
};

export default Spending;

const ProposalLine = (props: any) => {
  const { id, title, detail, author, executed, result } = props;
  const failed = executionFailed(result, executed);
  return (
    <div key={id} className={failed ? "bg-danger" : "bg-warn"}>
      <span
        className={`bg-${failed ? "danger" : "warning"} text-body p-1 mr-2`}
      >
        {detail ? amount(detail.spending[0]) : `?`} M
      </span>
      <Link to={`/proposals/${id}`}>{title}</Link> (
      <Link to={`/members/${author}`}>{author.handle}</Link>)
      {failed ? ` - ${failed}` : ""}
    </div>
  );
};
